%**************************************************************************
%
%   BENCHMARK STUDY ON RELIABILITY ESTIMATION IN HIGHER DIMENSIONS OF
%   STRUCTURAL SYSTEMS
%
%   Sample code for solving Problem 3 using direct Monte Carlo Simulation
%
%   Solution of Equation of Motion
%
%**************************************************************************
%   BECAUSE THE CODE IS PROVIDED FREE OF CHARGE, IT IS PROVIDED "AS IS" AND
%   WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED. 
%**************************************************************************

function [u]    = SEM3(n_dof,n_steps,Dt,a,struct_parameters)

%% 1.   Structural parameters
M   = struct_parameters(:,3);   %mass
Kd  = struct_parameters(:,1);   %stifness
R   = struct_parameters(:,2);   %stifness
S_K = struct_parameters(:,5);   %s/k
Z   = struct_parameters(:,4);   %damping

%% 2.   Definition of some parameters
S   = S_K.*Kd;
C   =  (2*Z.*sqrt(M.*(Kd+R)))';  % damping constant
p   = -M*a;         %equivalent force
M   = diag(M);      %mass matrix
C   = ckmatrix3(C);  %damping matrix
u0  = zeros(n_dof,1);   %initial condition
ud0 = zeros(n_dof,1);   %initial condition
kTdiag  = zeros(1,n_dof);

%% 3.   Initialize hysteresis loop
param   = cell(1,n_dof);
for w=1:n_dof
    param{w}    = restoringforce(Kd(w),R(w),S(w)); %Initialize hysteresis loop
end;

%% 4.   Parameters - numerical integration
gamma   = 0.5;      %Average acceleration method
beta    = 0.25;     %Average acceleration method
bdt     = beta*Dt;
g_b     = gamma/beta;
g_2b    = g_b/2;
g_bdt   = g_b/Dt;
dt2     = Dt*Dt;
bdt2    = beta*dt2;
one_bdt2    = 1/bdt2;
one_2b      = 1/(2*beta);

%% 5.   Matrices to store results of time integration
u       = zeros(n_dof,n_steps);
ud      = zeros(n_dof,n_steps);
udd     = zeros(n_dof,n_steps);
fs      = zeros(n_dof,n_steps);
resfor  = zeros(n_dof,n_steps);
ru      = zeros(n_dof,n_steps);

%% 6.   Initial conditions
u(:,1)  = u0;
ud(:,1) = ud0;
for w=1:n_dof
    [resfor(w,1), kTdiag(w), param{w}] = restoringforce(u0(w),0,ud0(w),0,0,param{w});
end;
fs(:,1) = [-diff(resfor(:,1)); resfor(n_dof,1)];
kT      = ckmatrix3(kTdiag);
udd(:,1)    = M\(p(:,1) - C*ud0 - fs(:,1));
A           = (1/bdt)*M + g_b*C;
B           = (1/(2*beta))*M + Dt*(g_2b-1)*C;
kgT         = kT + g_bdt*C + one_bdt2*M;
errr        = zeros(n_dof,n_steps);
period      = zeros(n_dof,n_steps);
period(:,1) = (2*pi)./sqrt(eig(full(kT),full(M)));
fsj         = zeros(n_dof,1);

%% 7.   Time integration
for i = 1:(n_steps-1)
    ui      = u(:,i);
    udi     = ud(:,i);
    uddi    = udd(:,i);
    dpi     = p(:,i+1) - p(:,i);
    dpgi    = dpi + A*udi + B*uddi;
    %Solve for duj from kgi and dpgi using Newton-Raphson
    %BEGIN NEWTON RAPHSON
    resfor0 = resfor(:,i);
    fs0 = fs(:,i);
    dR = dpgi;
    j = 0;
    duj = zeros(n_dof,1);
    ru0 = [ui(1);  diff(ui)];
    rv0 = [udi(1); diff(udi)];
    while 1
        j = j + 1;
        if j>10000
            error('Newton Raphson is not converging');
        end;
        dujold  = duj;
        duj     = kgT\dR;
        dudj    = g_bdt*duj - g_b*udi + Dt*(1-g_2b)*uddi;
        uj      = ui  + duj;
        vj      = udi + dudj;
        ruj     = [uj(1); diff(uj)];
        rvj     = [vj(1); diff(vj)];
        for w=1:n_dof
            [resfor(w,i+1), kTdiag(w), param{w}] = restoringforce(ruj(w),ru0(w),rvj(w),rv0(w),resfor0(w),param{w});
        end;
        fsj     = [-diff(resfor(:,i+1)); resfor(n_dof,i+1)];
        kT      = ckmatrix3(kTdiag);
        kgT     = kT + g_bdt*C + one_bdt2*M;
        df      = (fsj - fs0) - kT*duj;
        dR      = dpgi - df;
        if ((norm(duj-dujold) < 1e-12) || norm(df)<1e-12),
            break,
        end;
    end;
    fs(:,i+1)   = fsj;
    ru(:,i+1)   = ruj;
    %END NEWTON RAPHSON
    duddi       = (1/bdt2)*duj - (1/bdt)*udi - one_2b*uddi;
    u(:,i+1)    = uj;
    ud(:,i+1)   = vj;
    udd(:,i+1)  = uddi + duddi;
    errr(:,i+1) = M*udd(:,i+1) + C*ud(:,i+1) + fs(:,i+1) - p(:,i+1);
    period(:,i+1)   = (2*pi)./sqrt(eig(full(kT),full(M)));
end;

u   = u';

return